package org.bitbucket.bradleysmithllc.etlunit.feature.informatica;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaConfiguration;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaDomain;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaRepository;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaRepositoryClient;
import org.bitbucket.bradleysmithllc.etlunit.*;
import org.bitbucket.bradleysmithllc.etlunit.context.VariableContext;
import org.bitbucket.bradleysmithllc.etlunit.feature.informatica.json.informatica.create_informatica_connection.CreateInformaticaConnectionRequest;
import org.bitbucket.bradleysmithllc.etlunit.feature.informatica.json.informatica.create_informatica_folder.CreateInformaticaFolderRequest;
import org.bitbucket.bradleysmithllc.etlunit.feature.informatica.json.informatica.delete_informatica_connection.DeleteInformaticaConnectionRequest;
import org.bitbucket.bradleysmithllc.etlunit.feature.informatica.json.informatica.delete_informatica_folder.DeleteInformaticaFolderRequest;
import org.bitbucket.bradleysmithllc.etlunit.feature.informatica.json.informatica.execute.ExecuteHandler;
import org.bitbucket.bradleysmithllc.etlunit.feature.informatica.json.informatica.execute.ExecuteRequest;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestMethod;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestOperation;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestValueObject;
import org.bitbucket.bradleysmithllc.etlunit.util.IOUtils;

import javax.inject.Inject;
import javax.inject.Named;
import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.Map;

public class InformaticaClassListener extends NullClassListener implements ExecuteHandler
{
	private final InformaticaFeatureModule informaticaFeatureModule;
	private final InformaticaExecutor informaticaExecutor;

	private Log applicationLog;

	public InformaticaClassListener(InformaticaFeatureModule informaticaFeatureModule, InformaticaExecutor exe) {
		this.informaticaFeatureModule = informaticaFeatureModule;
		informaticaExecutor = exe;
	}

	@Override
	public void begin(ETLTestMethod mt, VariableContext context) throws TestAssertionFailure, TestExecutionError, TestWarning {
		// before each method, perform a deep cleanse of the working root
		InformaticaConfiguration config = informaticaFeatureModule.getInformaticaConfiguration();

		Map<String, InformaticaDomain> domMap = config.getDomains();

		Collection<InformaticaDomain> domSet = domMap.values();

		for (InformaticaDomain domain : domSet)
		{
			File workingRoot = domain.getLocalizedWorkingRoot();

			if (workingRoot.exists())
			{
				applicationLog.info("Purging remote workspace at path: " + workingRoot);
				IOUtils.purge(workingRoot, true);
				applicationLog.info("Done");
			}
		}
	}

	@Inject
	public void setLogger(@Named("applicationLog") Log logger)
	{
		this.applicationLog = logger;
	}

	public action_code process(ETLTestMethod mt, ETLTestOperation etlTestOperation, ETLTestValueObject etlTestValueObject, VariableContext context, ExecutionContext econtext) throws TestAssertionFailure, TestExecutionError, TestWarning {
		InformaticaRepository informaticaRepository = informaticaFeatureModule.repositoryForRequest(etlTestValueObject);

		if (etlTestOperation.getOperationName().equals("createInformaticaFolder"))
		{
			CreateInformaticaFolderRequest req = getRequestObject(etlTestValueObject, CreateInformaticaFolderRequest.class);
			InformaticaRepositoryClient ipc = informaticaRepository.getInformaticaRepositoryClient();

			try {
				ipc.createFolder(req.getFolder());
			} catch (Exception e) {
				applicationLog.severe("Failed to execute Informatica Command", e);
				throw new TestExecutionError("Failed to execute Informatica Command", InformaticaConstants.ERR_CREATE_FOLDER, e);
			}

			return action_code.handled;
		}

		if (etlTestOperation.getOperationName().equals("deleteInformaticaFolder"))
		{
			DeleteInformaticaFolderRequest req = getRequestObject(etlTestValueObject, DeleteInformaticaFolderRequest.class);
			InformaticaRepositoryClient ipc = informaticaRepository.getInformaticaRepositoryClient();

			try {
				ipc.deleteFolder(req.getFolder());
			} catch (Exception e) {
				throw new TestExecutionError("Failed to execute Informatica Command", InformaticaConstants.ERR_DELETE_FOLDER, e);
			}

			return action_code.handled;
		}

		if (etlTestOperation.getOperationName().equals("createInformaticaConnection"))
		{
			CreateInformaticaConnectionRequest req = getRequestObject(etlTestValueObject, CreateInformaticaConnectionRequest.class);
			InformaticaRepositoryClient ipc = informaticaRepository.getInformaticaRepositoryClient();

			try {
				ipc.createConnection(
					req.getServerName(),
					req.getDatabaseName(),
					req.getDatabaseUserName(),
					req.getDatabasePassword(),
					req.getRelationalConnectionName()
				);
			} catch (Exception e) {
				throw new TestExecutionError("Failed to execute Informatica Command", InformaticaConstants.ERR_CREATE_CONNECTION, e);
			}

			return action_code.handled;
		}

		if (etlTestOperation.getOperationName().equals("deleteInformaticaConnection"))
		{
			DeleteInformaticaConnectionRequest req = getRequestObject(etlTestValueObject, DeleteInformaticaConnectionRequest.class);
			InformaticaRepositoryClient ipc = informaticaRepository.getInformaticaRepositoryClient();

			try {
				ipc.deleteConnection(
					req.getRelationalConnectionName()
				);
			} catch (Exception e) {
				throw new TestExecutionError("Failed to execute Informatica Command", InformaticaConstants.ERR_DELETE_CONNECTION, e);
			}

			return action_code.handled;
		}

		return action_code.defer;
	}

	@Override
	public action_code execute(ExecuteRequest request, ETLTestMethod mt, ETLTestOperation testOperation, ETLTestValueObject valueObject, VariableContext variableContext, ExecutionContext executionContext)
			throws TestAssertionFailure, TestExecutionError, TestWarning
	{
		return informaticaExecutor.execute(request, mt, testOperation, valueObject, variableContext, executionContext);
	}

	public <T> T getRequestObject(ETLTestValueObject obj, Class<T> type) throws TestExecutionError {
		ObjectMapper objm = new ObjectMapper();

		try {
			String content = obj.getJsonNode().toString();

			applicationLog.info("Deserializing request object: " + content + " into type " + type);

			return objm.readValue(content, type);
		} catch (IOException e) {
			throw new TestExecutionError("Error converting object to request", e);
		}
	}
}